votTotalState = {};
source(VariableOpeningTime.modDir.."mohFeatures/votTotalStateOutput.lua");

function votTotalState:defaultValues()
	votTotalState.values = {objects={}, updateTimer=30, msec=false, group={productionPoint=false,animatedObject=false,animatedMapObject=false,sellingStation=false,buyingStation=false,beehive=false,animatedFenceObject=false}}; --not relevant for xml save
end;

function votTotalState:update(dt)
	if g_currentMission.multiOverlayV4.moduleFunction.getActiveRunningModul( {modName="variableOpeningTime", modulName="votTotalState"} ) then
		if votTotalState.values.updateTimer > 0 then			
			if g_currentMission.hl.timers["votTotalState"] ~= nil then return;end;
			g_currentMission.hl.addTimer( {delay=votTotalState.values.updateTimer, name="votTotalState", repeatable=1, ms=votTotalState.values.msec, action=votTotalState.load} ); 
		else
			votTotalState:load();
		end;	
	end;
end;

function votTotalState.loadOnStart(update) --onStartMission Callback, update = updateByPlayer !new object or delete object etc. delete your old cmds!	
	if votTotalState.values == nil then votTotalState:defaultValues();end;	
	if update == nil then votTotalState:loadAllState(true);end;	
end;

function votTotalState.load()	
	votTotalState:loadAllState();
end;

function votTotalState:setInfo(object, infoTable)		
	for a=1, #object.votOpeningTime do
		local openingTime = object.votOpeningTime[a];			
		infoTable.txt[a] = tostring(openingTime.startTime).. "-".. tostring(openingTime.endTime);				
	end;
	infoTable.nextActiveOpeningTime = object.votNextActiveOpeningTime;
	infoTable.activeOpeningTime = object.votActiveOpeningTime;
	infoTable.infoWhen = tostring(object.votNextHour.infoWhen.title).. " ".. tostring(object.votNextHour.infoWhen.text);
	infoTable.infoWhenVisible = object.votNextHour.infoWhen.visible;
	infoTable.isOpening = object.votIsOpening;
	infoTable.typ = object.votTyp;
end;

function votTotalState:loadAllState(generateCmd)	
	local isGroupVisible = false;
	for group, visible in pairs(votTotalState.values.group) do
		isGroupVisible = visible;
		if isGroupVisible then break;end;
	end;
	if isGroupVisible or generateCmd ~= nil then
		local playerFarmId = g_currentMission.player.farmId;	
		local votObjects = {};
		local objectId = {};
		local objects = g_currentMission.storageSystem:getLoadingStations();
		local showObject = nil;
		function setInfo(node)
			votSetGet:votIsOpening(showObject);
			if votObjects[showObject.votTyp] == nil then votObjects[showObject.votTyp] = {};end;
			local name = "Unknown";
			if showObject.getName ~= nil then name = showObject:getName();
			elseif showObject.saveId ~= nil then name = showObject.saveId;
				if string.find(name, "AnimatedObject_") then name = "Unknown Door";end;
			end;
			votObjects[showObject.votTyp][#votObjects[showObject.votTyp]+1] = {name=name or "Unknown",node=node,txt={}};
			votTotalState:setInfo(showObject, votObjects[showObject.votTyp][#votObjects[showObject.votTyp]]);		
		end;	
		if objects ~= nil then
			for _, object in pairs(objects) do
				local isVotObject, votObject = votSetGet:getVotObject(object);
				if isVotObject then
					local node = votObject.rootNode or votObject.nodeId or votObject.node;
					if node ~= nil and objectId[node] == nil then
						objectId[node] = true;
						showObject = votObject;
						setInfo(node);					
					end;
				end;
			end;
		end;
		objects = g_currentMission.storageSystem:getUnloadingStations();
		if objects ~= nil then
			for _, object in pairs(objects) do
				local isVotObject, votObject = votSetGet:getVotObject(object);
				if isVotObject then
					local node = votObject.rootNode or votObject.nodeId or votObject.node;
					if node ~= nil and objectId[node] == nil and votObject.typ ~= "productionPoint" then
						objectId[node] = true;
						showObject = votObject;
						setInfo(node);					
					end;
				end;
			end;
		end;
		if VariableOpeningTime.objects ~= nil then
			if VariableOpeningTime.objects.beehive ~= nil and #VariableOpeningTime.objects.beehive > 0 then
				for a=1, #VariableOpeningTime.objects.beehive do					
					local isVotObject, votObject = votSetGet:getVotObject(VariableOpeningTime.objects.beehive[a]);
					if isVotObject then
						local node = votObject.rootNode or votObject.nodeId or votObject.node;
						if node ~= nil and objectId[node] == nil then
							objectId[node] = true;
							showObject = votObject;
							setInfo(node);					
						end;					
					end;
				end;			
			end;	
			if VariableOpeningTime.objects.animatedObject ~= nil and #VariableOpeningTime.objects.animatedObject > 0 then
				for a=1, #VariableOpeningTime.objects.animatedObject do
					local isVotObject, votObject = votSetGet:getVotObject(VariableOpeningTime.objects.animatedObject[a]);
					if isVotObject then
						local node = votObject.nodeId or votObject.node or votObject.rootNode or votObject.saveId;
						if node ~= nil and objectId[node] == nil then
							objectId[node] = true;
							showObject = votObject;
							setInfo(node);					
						end;
					end;
				end;			
			end;		
			if VariableOpeningTime.objects.animatedMapObject ~= nil and #VariableOpeningTime.objects.animatedMapObject > 0 then
				for a=1, #VariableOpeningTime.objects.animatedMapObject do
					local isVotObject, votObject = votSetGet:getVotObject(VariableOpeningTime.objects.animatedMapObject[a]);
					if isVotObject then
						local node = votObject.nodeId or votObject.node or votObject.rootNode or votObject.saveId;
						if node ~= nil and objectId[node] == nil then
							objectId[node] = true;
							showObject = votObject;
							setInfo(node);					
						end;					
					end;
				end;			
			end;		
			if VariableOpeningTime.objects.animatedFenceObject ~= nil and #VariableOpeningTime.objects.animatedFenceObject > 0 then
				for a=1, #VariableOpeningTime.objects.animatedFenceObject do
					local isVotObject, votObject = votSetGet:getVotObject(VariableOpeningTime.objects.animatedFenceObject[a]);
					if isVotObject then
						local node = votObject.nodeId or votObject.node or votObject.rootNode or votObject.saveId;
						if node ~= nil and objectId[node] == nil then
							objectId[node] = true;
							showObject = votObject;
							setInfo(node);					
						end;					
					end;
				end;			
			end;		
			function comp(w1,w2)						
				if w1.name < w2.name then
					return true;
				end;				
			end;
			for k, v in pairs(votObjects) do
				table.sort(v, comp)
			end;
			votTotalState.values.objects = votObjects; --new
		end;
	end;
	if generateCmd ~= nil and generateCmd then votTotalState:generateCmd();end;
end;

function votTotalState:generateCmd()
	g_currentMission.multiOverlayV4.cmdFunction.generateCmd( { 
			modName = "VariableOpeningTime";
			cmdName = "output_votTotalState_";
			groupName = "OtherMods";			
			displayName = g_i18n:getText("ui_operatingHours").. "/".. g_i18n:getText("vot_WorkingHours").. "/".. g_i18n:getText("vot_OpeningTime").. "(VOT Mod)";
			infoTxt = "Click Left/Right/Middle for Interaction.";
			mouseClickAccepts = {true,true,true};
			ownTable = {};
			typ = {"output_"};			
			actionCallback = votTotalStateOutput.giveOutputTable;			
		}
	);	
end;

if g_currentMission.multiOverlayV4 ~= nil then
	g_currentMission.multiOverlayV4.moduleFunction.addCallbackModul( { modName="variableOpeningTime", modulCallbackName="onStartMission", callback=votTotalState.loadOnStart } );
	g_currentMission.multiOverlayV4.moduleFunction.addCallbackModul( { modName="variableOpeningTime", modulCallbackName="update", callback=votTotalState.update } );
end;